<?php

declare(strict_types=1);

namespace App\Service\Persist;

use App\Entity\Remote\Brilo\Posts\Post;
use App\Repository\Post\PostRepository;
use App\Repository\Users\UserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Psr\Log\LoggerInterface;

class RemotePostsConvertor
{
    public function __construct(
        protected readonly PostRepository $postRepository,
        protected readonly UserRepository $userRepository,
        protected readonly LoggerInterface $logger,
    ) {
    }

    /**
     * Convert remote posts into database posts.
     * @param Post[] $remotePosts
     * @return \Generator<\App\Entity\Database\Posts\Post>
     */
    public function convert(iterable $remotePosts): \Generator
    {
        foreach ($remotePosts as $remotePost) {
            $post = $this->postRepository->find($remotePost->id);
            if ($post === null) {
                $this->logger->info("Creating new post for ID: {$remotePost->id}");
                yield $this->createPost($remotePost);
            } else {
                $this->logger->info("Updating post for ID: {$remotePost->id}");
                $this->updatePost($remotePost, $post);
                yield $post;
            }
        }
    }

    /**
     * Create a new post from the remote data.
     * @param Post $remotePost
     * @return \App\Entity\Database\Posts\Post
     */
    protected function createPost(Post $remotePost): \App\Entity\Database\Posts\Post
    {
        $user = $this->userRepository->find($remotePost->userId) ?? throw new \LogicException("User not found for ID: {$remotePost->userId}");
        return new \App\Entity\Database\Posts\Post(
            $remotePost->id,
            $user,
            $remotePost->title,
            $remotePost->body,
            new ArrayCollection(),
        );
    }

    /**
     * Update an existing post with the remote data.
     * @param Post $remotePost
     * @param \App\Entity\Database\Posts\Post $post
     * @return void
     */
    protected function updatePost(Post $remotePost, \App\Entity\Database\Posts\Post $post): void
    {
        $user = $this->userRepository->find($remotePost->userId) ?? throw new \LogicException("User not found for ID: {$remotePost->userId}");
        $post->user = $user;
        $post->title = $remotePost->title;
        $post->body = $remotePost->body;
    }
}
