<?php

declare(strict_types=1);

namespace App\Service\Persist;

use App\Entity\Database\Users\Address;
use App\Entity\Database\Users\Company;
use App\Entity\Remote\Brilo\Users\User;
use App\Repository\Users\CompanyRepository;
use App\Repository\Users\UserRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Psr\Log\LoggerInterface;

class RemoteUsersConvertor
{
    public function __construct(
        protected readonly UserRepository $userRepository,
        protected readonly CompanyRepository $companyRepository,
        protected readonly LoggerInterface $logger,
    ) {
    }

    /**
     * Converts remote users to database users
     * @param User[] $remoteUsers
     * @return \Generator<\App\Entity\Database\Users\User>
     */
    public function convert(iterable $remoteUsers): \Generator
    {
        $ids = array_map(function ($remoteUser) {
            return $remoteUser->id;
        }, (array) $remoteUsers);

        $existentIds = $this->userRepository->findByIds($ids);

        foreach ($remoteUsers as $remoteUser) {
            if (isset($existentIds[$remoteUser->id])) {
                $localUser = $existentIds[$remoteUser->id];
                $this->updateUser($remoteUser, $localUser);
                $this->logger->debug("Updating user with ID: {$remoteUser->id}");
                yield $localUser;
            } else {
                $this->logger->debug("Creating new user with ID: {$remoteUser->id}");
                yield $this->createUser($remoteUser);
            }
        }
    }

    /**
     * Update user from the given remote user
     * @param User $remoteUser
     * @param \App\Entity\Database\Users\User $user
     * @return void
     */
    protected function updateUser(User $remoteUser, \App\Entity\Database\Users\User $user): void
    {
        $user->name = $remoteUser->name;
        $user->username = $remoteUser->username;
        $user->email = $remoteUser->email;
        $user->phone = $remoteUser->phone;
        $user->website = $remoteUser->website;
        $user->address->city = $remoteUser->address->city;
        $user->address->street = $remoteUser->address->street;
        $user->address->suite = $remoteUser->address->suite;
        $user->address->zipcode = $remoteUser->address->zipcode;
        $user->address->lat = (float) $remoteUser->address->geo->lat;
        $user->address->lng = (float) $remoteUser->address->geo->lng;
        if ($user->company->name !== $remoteUser->company->name) {
            $user->company = $this->companyRepository->findByName($remoteUser->company->name) ?? new Company(
                null,
                new ArrayCollection(),
                $remoteUser->company->name,
                $remoteUser->company->catchPhrase,
                $remoteUser->company->bs,
            );
        } else {
            $user->company->catchPhrase = $remoteUser->company->catchPhrase;
            $user->company->bs = $remoteUser->company->bs;
        }
    }

    /**
     * Creates a new user from the given remote user
     * @param User $remoteUser
     * @return \App\Entity\Database\Users\User
     */
    protected function createUser(User $remoteUser): \App\Entity\Database\Users\User
    {
        return new \App\Entity\Database\Users\User(
            $remoteUser->id,
            $remoteUser->name,
            $remoteUser->username,
            $remoteUser->email,
            $remoteUser->phone,
            $remoteUser->website,
            new Address(
                null,
                null,
                $remoteUser->address->street,
                $remoteUser->address->suite,
                $remoteUser->address->city,
                $remoteUser->address->zipcode,
                (float) $remoteUser->address->geo->lat,
                (float) $remoteUser->address->geo->lng,
            ),
            $this->companyRepository->findByName($remoteUser->company->name) ?? new Company(
                null,
                new ArrayCollection(),
                $remoteUser->company->name,
                $remoteUser->company->catchPhrase,
                $remoteUser->company->bs,
            )
        );
    }
}
